package com.dsic.gj_erp.service.sy;


import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.dsic.gj_erp.bean.sy.*;
import com.dsic.gj_erp.dao.sy.*;
import com.dsic.gj_erp.mapper.SyuserMapper;
import com.dsic.gj_erp.util.MD5Util;
import com.querydsl.core.Tuple;
import com.querydsl.core.types.dsl.BooleanExpression;
import com.querydsl.jpa.impl.JPAQueryFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestBody;

import javax.annotation.PostConstruct;
import javax.persistence.EntityManager;
import javax.xml.bind.DatatypeConverter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.alibaba.excel.util.StringUtils.isEmpty;

/**
 * 系统安全子系统处理逻辑
 */
@Service
public class SYService {

    @Autowired
    Dm_zhbmlbpRepository dm_zhbmlbpRepository;
    @Autowired
    Dm_zhbmpRepository dm_zhbmpRepository;
    @Autowired
    SyworkgpRepository syworkgpRepository;
    @Autowired
    SyuirpRepository syuirpRepository;
    @Autowired
    SyigrpRepository syigrpRepository;
    @Autowired
    SyimrpRepository syimrpRepository;
    @Autowired
    SyifrpRepository syifrpRepository;
    @Autowired
    SyuserpRepository syuserpRepository;
    @Autowired
    SyfunpRepository syfunpRepository;
    @Autowired
    DmBhdypRepository dmBhdypRepository;
    @Autowired
    DmBhsjpRepository dmBhsjpRepository;
    @Autowired
    DmBhzybRepository dmBhzybRepository;
    @Autowired
    private EntityManager entityManager;
    @Autowired
    private SyuserMapper syuserMapper;
    private JPAQueryFactory queryFactory;

    @PostConstruct
    public void initFactory() {
        queryFactory = new JPAQueryFactory(entityManager);
    }

    public List<Syworkgp> getWorkGroupList(String yhdm) {
        QSyuirp syuirp = QSyuirp.syuirp;
        QSyworkgp syworkgp = QSyworkgp.syworkgp;
        if (StringUtils.isEmpty(yhdm)) {
            return queryFactory.select(syworkgp).from(syworkgp).orderBy(syworkgp.JKDBM.asc()).fetch();
        } else {
            return queryFactory.select(syworkgp).from(syworkgp).leftJoin(syuirp).on(syuirp.GWDM.eq(syworkgp.JKDBM)).where(syuirp.YHDM.eq(yhdm)).orderBy(syworkgp.JKDBM.asc()).where().fetch();
        }
    }

    public Syworkgp addWordGroup(Syworkgp syworkgp) {
        return syworkgpRepository.save(syworkgp);
    }

    @Transactional
    public void deleteWorkGroup(Syworkgp syworkgp) {
        syuirpRepository.deleteByGWDM(syworkgp.getJKDBM());
        syigrpRepository.deleteByGWDM(syworkgp.getJKDBM());
        syimrpRepository.deleteByGWDM(syworkgp.getJKDBM());
        syifrpRepository.deleteByGWDM(syworkgp.getJKDBM());
        syworkgpRepository.deleteById(syworkgp.getJKDBM());
    }

    @Transactional
    public Syworkgp updateWorkGroup(Syworkgp osyworkgp, Syworkgp nsyworkgp) {
        QSyworkgp syworkgp = QSyworkgp.syworkgp;
        QSyuirp syuirp = QSyuirp.syuirp;
        QSyigrp syigrp = QSyigrp.syigrp;
        QSyimrp syimrp = QSyimrp.syimrp;
        QSyifrp syifrp = QSyifrp.syifrp;
        String newgwdm = nsyworkgp.getJKDBM();
        String gwdm = osyworkgp.getJKDBM();
        queryFactory.update(syuirp).set(syuirp.GWDM, newgwdm).where(syuirp.GWDM.eq(gwdm)).execute();
        queryFactory.update(syigrp).set(syigrp.GWDM, newgwdm).where(syigrp.GWDM.eq(gwdm)).execute();
        queryFactory.update(syimrp).set(syimrp.GWDM, newgwdm).where(syimrp.GWDM.eq(gwdm)).execute();
        queryFactory.update(syifrp).set(syifrp.GWDM, newgwdm).where(syifrp.GWDM.eq(gwdm)).execute();
        syworkgpRepository.delete(osyworkgp);
        return syworkgpRepository.save(nsyworkgp);
    }


    public Map<String, List<Dm_zhbmp>> getUserZHBM() {
        Map<String, List<Dm_zhbmp>> dataMap = new HashMap<>();
        dataMap.put("yhlx", dm_zhbmpRepository.findByBMLB("YHLX"));
        dataMap.put("spqx", dm_zhbmpRepository.findByBMLB("SPQX"));
        List<Dm_zhbmp> bmbmList = new ArrayList<>();
        for (Map map : dm_zhbmpRepository.findBMBM()) {
            Dm_zhbmp dm_zhbmp = new Dm_zhbmp();
            dm_zhbmp.setBM((String) map.get("BM"));
            dm_zhbmp.setBmsm((String) map.get("MC"));
            bmbmList.add(dm_zhbmp);
        }
        dataMap.put("bmbm", bmbmList);
        List<Dm_zhbmp> zxtBmbmList = new ArrayList<>();
        for (Sysysp sysysp : getZxtList(null)) {
            Dm_zhbmp dm_zhbmp = new Dm_zhbmp();
            dm_zhbmp.setBM(sysysp.getZXTDM());
            dm_zhbmp.setBmsm(sysysp.getZXTMS());
            zxtBmbmList.add(dm_zhbmp);
        }
        dataMap.put("zxtbm", zxtBmbmList);
        return dataMap;
    }


    public List<Syuserp> getUserList(String gwdm, Integer sort) {
        QSyuirp syuirp = QSyuirp.syuirp;
        QSyuserp syuserp = QSyuserp.syuserp;
        List<Syuserp> userlist = new ArrayList<>();
        if (StringUtils.isEmpty(gwdm)) {
            userlist = queryFactory.select(syuserp).from(syuserp).where().orderBy(sort != 1 ? syuserp.YHDM.asc() : syuserp.BMBM.asc()).fetch();
        } else {
            userlist = queryFactory.select(syuserp).from(syuserp, syuirp).where(syuserp.YHDM.eq(syuirp.YHDM).and(syuirp.GWDM.eq(gwdm))).orderBy(syuserp.BMBM.asc()).fetch();
        }
        return userlist;
    }
    /**
     * 公共函数：取最大单据号（导入图纸时，生成主键流水号）
     * @param zxt 子系统编码
     * @param jg 机构码
     * @param bmm 编码名称
     * @return 最大单据号（流水号）
     */
//    @Transactional(propagation = Propagation.REQUIRES_NEW,rollbackFor = Exception.class)
    @Transactional
    public String f_getpjh(String zxt,String jg,String bmm){
        entityManager.clear(); // 清除缓存
        QDmBhdyp bhdy=QDmBhdyp.dmBhdyp;
        List<DmBhdyp> bhdyps;
        //确定编码前缀值开始
        bhdyps = queryFactory.select(bhdy).from(bhdy)
                                            .where(
                                                    bhdy.zxtbm.eq(zxt)
                                                    .and(bhdy.jgm.eq(jg))
                                                    .and(bhdy.bmmc.eq(bmm)))
                                                    .orderBy(bhdy.bmsx.asc())
                                                    .fetch();
        String f_qz=bhdyps.get(0).getBmqz().trim(); //
        String f_sx=bhdyps.get(0).getBmsx().trim(); //

        if(StringUtils.isEmpty(f_qz)){
            new  Exception("不合要求的错误数据");
        }else {
            if (f_qz.equals("年")){
                f_qz= DateUtil.format(DateUtil.date(),"yy");
            }
            if (f_qz.equals("年月")){
                f_qz= DateUtil.format(DateUtil.date(),"yyMM");
            }

            if (f_qz.equals("年月日")){
                f_qz= DateUtil.format(DateUtil.date(),"yyMMdd");
            }
        }
        //确定编码前缀值结束

        //生成单据号开始
        QDmBhsjp bhsjp=QDmBhsjp.dmBhsjp;
        List<DmBhsjp> bhsjps;
        bhsjps = queryFactory.select(bhsjp).from(bhsjp).where(
                                                            bhsjp.zxtbm.eq(zxt)
                                                            .and(bhsjp.jgm.eq(jg))
                                                            .and(bhsjp.bmmc.eq(bmm))
                                                            .and(bhsjp.bmqz.eq(f_qz)))
                                                            .fetch();

        if (ObjectUtil.isNotEmpty(bhsjps)){
            String f_sxh=bhsjps.get(0).getBmsxh().trim();
            f_sxh=(Integer.parseInt(f_sxh)+1)+"";
            String f_nxh=f_sxh;
            //使单据号满十位
            if(f_sxh.length()<10-f_qz.length()){
                int num=10-f_qz.length()-f_sxh.length();
                for (int i = 0; i < num; i++) {
                    f_nxh='0' +f_nxh;
                }
            }else {
                f_nxh= f_nxh.substring(f_nxh.length() - (10 - f_qz.length()));
            }
            queryFactory.update(bhsjp).set(bhsjp.bmsxh, f_nxh).where(bhsjp.zxtbm.eq(zxt)
                                                                    .and(bhsjp.jgm.eq(jg))
                                                                    .and(bhsjp.bmmc.eq(bmm))
                                                                    .and(bhsjp.bmqz.eq(f_qz))
                                                            ).execute();

            if(bmm.equals("APS")){
                return f_qz+f_nxh;
            }
            DmBhzyb bhzyb=new DmBhzyb();
            bhzyb.setBhzyb(zxt +jg+bmm+f_qz+f_nxh);
            dmBhzybRepository.save(bhzyb);
            return f_qz+f_nxh;
        }else {
           String f_oxh=f_sx;

            if (f_sx.length()<10-f_qz.length()){
                for (int i = 0; i < 10-f_qz.length()-f_sx.length(); i++) {
                    f_oxh='0' +f_oxh;
                }
            }else {
                f_oxh=f_oxh.substring(f_oxh.length()-(10-f_qz.length()));
            }

            DmBhsjp dmBhsjp=new DmBhsjp();
            dmBhsjp.setZxtbm(zxt);
            dmBhsjp.setJgm(jg);
            dmBhsjp.setBmmc(bmm);
            dmBhsjp.setBmqz(f_qz);
            dmBhsjp.setBmsxh(f_oxh);
            dmBhsjpRepository.addBhsjp(dmBhsjp);
            if (bmm.equals("APS")){
                return f_qz+f_oxh;
            }
            DmBhzyb bhzyb=new DmBhzyb();
            bhzyb.setBhzyb(zxt +jg+bmm+f_qz+f_oxh);
            dmBhzybRepository.save(bhzyb);

            return f_qz+f_oxh;
        }
    }

    public List<Map<String, Object>> getUserListByFunc(Syifrp syifrpBean) {
        QSyuserp syuserp = QSyuserp.syuserp;
        QSyuirp syuirp = QSyuirp.syuirp;
        QSyifrp syifrp = QSyifrp.syifrp;
        List<Tuple> tupleList = queryFactory.select(syuserp.BMBM, syuirp.YHDM, syuserp.YHMS, syuirp.GWDM).from(syifrp, syuirp, syuserp).where(syifrp.GWDM.eq(syuirp.GWDM).and(syuirp.YHDM.eq(syuserp.YHDM).and(syifrp.ZXTBM.eq(syifrpBean.getZXTBM()).and(syifrp.MS.eq(syifrpBean.getMS()))))).fetch();
        List<Map> bmList = dm_zhbmpRepository.findBMBM();
        List<Map<String, Object>> resultList = new ArrayList<Map<String, Object>>();
        for (Tuple tuple : tupleList) {
            List<Map> bms = bmList.stream().filter((e) -> e.get("BM").equals(tuple.get(0, String.class))).collect(Collectors.toList());
            String bmmc = bms.size() > 0 ? (String) bms.get(0).get("MC") : "";
            String yhdm = tuple.get(1, String.class);
            String yhms = tuple.get(2, String.class);
            String gwdm = tuple.get(3, String.class);
            List<Map<String, Object>> userList = resultList.stream().filter((e) -> e.get("yhdm").equals(yhdm)).collect(Collectors.toList());
            if (userList.size() > 0) {
                userList.get(0).put("gwdm", userList.get(0).get("gwdm") + ";" + gwdm);
            } else {
                HashMap<String, Object> dataMap = new HashMap<>();
                dataMap.put("bmmc", bmmc);
                dataMap.put("yhdm", yhdm);
                dataMap.put("yhms", yhms);
                dataMap.put("gwdm", gwdm);
                resultList.add(dataMap);
            }
        }
        return resultList;
    }

    public Syuserp addUser(Syuserp syuserp) {
//        syuserp.setPWD(MD5Util.encrypt(syuserp.getPWD()));
        syuserp.setPWD("");
        return syuserpRepository.save(syuserp);
    }

    public void deleteUser(Syuserp syuserp) {
        syuserpRepository.delete(syuserp);
    }

    @Transactional
    public void updateUser(Syuserp syuserp) {
//        QSyuirp syuirp = QSyuirp.syuirp;
//        queryFactory.update(syuirp).set(syuirp.YHDM, nsyuserp.getYHDM()).where(syuirp.YHDM.eq(osyuserp.getYHDM())).execute();
//        syuserpRepository.delete(osyuserp);
        syuserMapper.updateUser(syuserp);
//        return syuserpRepository.save(nsyuserp);
    }

    public void addWorkGroupUser(@RequestBody List<Syuirp> syuirp) {
        syuirpRepository.saveAll(syuirp);
    }

    public void deleteWorkGroupUser(@RequestBody List<Syuirp> syuirp) {
        syuirpRepository.deleteAll(syuirp);
    }


    public List<Dm_zhbmp> getFzqxList(String gwdm) {
        QSyigrp syigrp = QSyigrp.syigrp;
        QDm_zhbmp dm_zhbmp = QDm_zhbmp.dm_zhbmp;
        BooleanExpression inSys = dm_zhbmp.BM.like("IM%").or(dm_zhbmp.BM.like("SD%").or(dm_zhbmp.BM.like("MS%"))
                .or(dm_zhbmp.BM.like("FP%")).or(dm_zhbmp.BM.like("HR%")).or(dm_zhbmp.BM.like("YS%")).or(dm_zhbmp.BM.like("DJ%")));
        if (StringUtils.isEmpty(gwdm)) {
            return queryFactory.select(dm_zhbmp).from(dm_zhbmp).where(dm_zhbmp.BMLB.eq("BMBM").and(inSys)).fetch();
        } else {
            return queryFactory.select(dm_zhbmp).from(syigrp).leftJoin(dm_zhbmp).on(dm_zhbmp.BM.eq(syigrp.ZXTDM.concat(syigrp.FZDM))).where(syigrp.GWDM.eq(gwdm).and(dm_zhbmp.BMLB.eq("BMBM")).and(inSys)).fetch();
        }
    }

    public void addWorkGroupFz(List<Syigrp> syigrp) {
        syigrpRepository.saveAll(syigrp);
    }

    public void deleteWorkGroupFz(@RequestBody List<Syigrp> syigrp) {
        syigrpRepository.deleteAll(syigrp);
    }

    public List<Sysysp> getZxtList(String gwdm) {
        QSyimrp syimrp = QSyimrp.syimrp;
        QSysysp sysysp = QSysysp.sysysp;
        if (StringUtils.isEmpty(gwdm)) {
            return queryFactory.select(sysysp).from(sysysp).orderBy(sysysp.YLZD1.asc()).fetch();
        } else {
            return queryFactory.select(sysysp).from(sysysp).leftJoin(syimrp).on(sysysp.ZXTDM.eq(syimrp.ZXTBM)).where(syimrp.GWDM.eq(gwdm)).orderBy(sysysp.YLZD1.asc()).fetch();
        }
    }

    public void addWorkGroupZxt(List<Syimrp> syimrp) {
        syimrpRepository.saveAll(syimrp);
    }

    public void deleteWorkGroupZxt(@RequestBody List<Syimrp> syimrp) {
        syimrpRepository.deleteAll(syimrp);
    }


    public List<Syfunp> getFuncList(Integer type, String gwdm) {
        QSyifrp syifrp = QSyifrp.syifrp;
        QSyfunp syfunp = QSyfunp.syfunp;
        QSysysp sysysp = QSysysp.sysysp;
        if (type != null && type == 1) {
            return queryFactory.select(syfunp).from(syfunp, sysysp).where(sysysp.ZXTDM.eq(syfunp.ZXTBM)).orderBy(sysysp.YLZD1.asc()).orderBy(syfunp.YLZD3.asc()).fetch();
        } else {
            if (StringUtils.isEmpty(gwdm)) {
                return queryFactory.select(syfunp).from(syfunp).where(syfunp.CKDM.ne("")).orderBy(syfunp.ZXTBM.asc()).orderBy(syfunp.YLZD2.asc()).orderBy(syfunp.YLZD3.asc()).fetch();
            } else {
                return queryFactory.select(syfunp).from(syfunp).leftJoin(syifrp).on(syfunp.MS.eq(syifrp.MS).and(syfunp.ZXTBM.eq(syifrp.ZXTBM))).where(syfunp.CKDM.ne("").and(syifrp.GWDM.eq(gwdm))).orderBy(syfunp.ZXTBM.asc()).orderBy(syfunp.YLZD2.asc()).orderBy(syfunp.YLZD3.asc()).fetch();
            }
        }
    }


    public Syfunp addFunc(Syfunp syfunp) {
        return syfunpRepository.save(syfunp);
    }


    public void deleteFunc(Syfunp syfunp) {
        syfunpRepository.delete(syfunp);
    }

    @Transactional
    public Syfunp updateFunc(Syfunp osyfunp, Syfunp nsyfunp) {
        QSyifrp syifrp = QSyifrp.syifrp;
        queryFactory.update(syifrp).set(syifrp.ZXTBM, nsyfunp.getZXTBM())
                .set(syifrp.MS, nsyfunp.getMS())
                .set(syifrp.GNLB, nsyfunp.getGNLB()).where(syifrp.ZXTBM.eq(osyfunp.getZXTBM()).and(syifrp.MS.eq(osyfunp.getMS()).and(syifrp.GNLB.eq(osyfunp.getGNLB())))).execute();
        syfunpRepository.delete(osyfunp);
        return syfunpRepository.save(nsyfunp);
    }


    public void addWorkGroupFunc(List<Syifrp> Syifrp) {
        syifrpRepository.saveAll(Syifrp);
    }

    public void deleteWorkGroupFunc(@RequestBody List<Syifrp> syifrp) {
        syifrpRepository.deleteAll(syifrp);
    }


    public List<Dm_zhbmlbp> getZhbmlbList(int sort) {
        if (sort == 0) {
            return dm_zhbmlbpRepository.findAll(Sort.by("LBMC").ascending());
        } else {
            return dm_zhbmlbpRepository.findAll(Sort.by("BMLB").ascending());
        }
    }

    public List<Dm_zhbmp> getZhbmList(Dm_zhbmlbp zhbmlbp) {
        return dm_zhbmpRepository.findByBMLB(zhbmlbp.getBMLB());
    }

    public Dm_zhbmp addZhbm(Dm_zhbmp zhbmp) {
        return dm_zhbmpRepository.save(zhbmp);
    }

    public void deleteZhbm(Dm_zhbmp zhbmp) {
        dm_zhbmpRepository.delete(zhbmp);
    }

    @Transactional
    public Dm_zhbmp updateZhbm(Dm_zhbmp ozhbmp, Dm_zhbmp nzhbmp) {
        dm_zhbmpRepository.delete(ozhbmp);
        return dm_zhbmpRepository.save(nzhbmp);

//        if(ozhbmp.getBMLB().equals("BMBM")){
//            QSyigrp syigrp =QSyigrp.syigrp;
//            queryFactory.update(syigrp).set(syigrp.FZDM, nzhbmp.getBM()).where(syigrp.ZXTDM.concat(syigrp.FZDM).eq(ozhbmp.getBM())).execute();
//        }
    }
    public List<Dm_zhbmp> getALL() {
        QDm_zhbmp dm_zhbmp=QDm_zhbmp.dm_zhbmp;
        List<Dm_zhbmp> fetch = queryFactory.select(dm_zhbmp).from(dm_zhbmp).fetch();
        return fetch;
    }

    public List<Map>  em_sbjbb_ZM(String bmlb){
        List<Map> dmZhbmp;
        if (bmlb.equals("bmbm")){
            dmZhbmp=dm_zhbmpRepository.findbmbm();
        }
        else if(bmlb.equals("QGKW")){
            dmZhbmp=dm_zhbmpRepository.findqgkw();
        }
        else {
            dmZhbmp=dm_zhbmpRepository.findjldw(bmlb);
        }
        return dmZhbmp;
    }
    public List<Map> getSystemList() {
        return syuserMapper.getSystemList();
    }
    public List<Map> getMenuList() {
        return syuserMapper.getMenuList();
    }
    public List<Map> getMenuForRole() {
        return syuserMapper.getMenuForRole();
    }

    public List<Map> getUserListByMenu(Map<String, Object> paramMap) {
        return syuserMapper.getUserListByMenu(paramMap);
    }

    public List<Map> getDepartList() {
        return syuserMapper.getDepartList();
    }

    public List<Map> getRoleList() {
        return syuserMapper.getRoleList();
    }

    public List<Map> getRoleListByUser(String user_code) {
        return syuserMapper.getRoleListByUser(user_code);
    }

    public void addRoleForUser(List<Map<String, Object>> list) {
        for (Map map : list) {
            syuserMapper.addRoleForUser(map);
        }
    }

    public void deleteRoleForUser(List<Map<String, Object>> list) {
        for (Map map : list) {
            syuserMapper.deleteRoleForUser(map);
        }
    }

    public List<Map> getInstitutionList() {
        return syuserMapper.getInstitutionList();
    }

    public List<Map> getUserListByRole(String role_code) {
        return syuserMapper.getUserListByRole(role_code);
    }


    public List<Map> getInstitutionListByRole(String role_code) {
        return syuserMapper.getInstitutionListByRole(role_code);
    }

    public List<Map> getSystemListByRole(String role_code) {
        return syuserMapper.getSystemListByRole(role_code);
    }
    public List<Map> getMenuListByRole(String role_code) {
        return syuserMapper.getMenuListByRole(role_code);
    }

    public Integer addRole(Map role) {
        return syuserMapper.addRole(role);
    }

    public Integer deleteRole(String role_code) {
        return syuserMapper.deleteRole(role_code);
    }

    public Integer updateRole(Map role) {
        return syuserMapper.updateRole(role);
    }

    public void addRoleForInstitution(List<Map<String, Object>> paramList) {
        for (Map map : paramList) {
            syuserMapper.addRoleForInstitution(map);
        }
    }

    public void deleteRoleForInstitution(List<Map<String, Object>> paramList) {
        for (Map map : paramList) {
            syuserMapper.deleteRoleForInstitution(map);
        }
    }


    public void addRoleForSystem(List<Map<String, Object>> paramList) {
        for (Map map : paramList) {
            syuserMapper.addRoleForSystem(map);
        }
    }

    public void deleteRoleForSystem(List<Map<String, Object>> paramList) {
        for (Map map : paramList) {
            syuserMapper.deleteRoleForSystem(map);
        }
    }

    public void addRoleForMenu(List<Map<String, Object>> paramList) {
        for (Map map : paramList) {
            syuserMapper.addRoleForMenu(map);
        }
    }

    public void deleteRoleForMenu(List<Map<String, Object>> paramList) {
        for (Map map : paramList) {
            syuserMapper.deleteRoleForMenu(map);
        }
    }

}
